<?php
// Own namespace
namespace Org\Mxchange\CoreFramework\Registration;

// Import framework stuff
use Org\Mxchange\CoreFramework\Bootstrap\FrameworkBootstrap;
use Org\Mxchange\CoreFramework\Factory\ObjectFactory;
use Org\Mxchange\CoreFramework\Filter\Filterable;
use Org\Mxchange\CoreFramework\Object\BaseFrameworkSystem;

/**
 * A general registration class.
 *
 * @author		Roland Haeder <webmaster@shipsimu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, 2009 - 2017 Core Developer Team
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.shipsimu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
abstract class BaseRegistration extends BaseFrameworkSystem {
	/**
	 * Pre-registration filter chain
	 */
	private $preRegistrationFilter = NULL;

	/**
	 * Pre-registration filter chain
	 */
	private $postRegistrationFilter = NULL;

	/**
	 * Protected constructor
	 *
	 * @param	$className	Name of the class
	 * @return	void
	 */
	protected function __construct ($className) {
		// Call parent constructor
		parent::__construct($className);
	}

	/**
	 * Initialize filters. This must be done before you can use them
	 *
	 * @return	void
	 */
	protected function initFilterChains () {
		// Pre/post-registration filters
		$this->preRegistrationFilter = ObjectFactory::createObjectByConfiguredName('filter_chain_class');
		$this->postRegistrationFilter = ObjectFactory::createObjectByConfiguredName('filter_chain_class');
	}

	/**
	 * Adds a filter to the pre filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPreFilter (Filterable $filterInstance) {
		// Add the pre filter
		$this->preRegistrationFilter->addFilter($filterInstance);
	}

	/**
	 * Adds a filter to the post filter chain
	 *
	 * @param	$filterInstance		An instance of a filter
	 * @return	void
	 */
	public function addPostFilter (Filterable $filterInstance) {
		// Add the post filter
		$this->postRegistrationFilter->addFilter($filterInstance);
	}

	/**
	 * Executes all pre filters
	 *
	 * @return	void
	 */
	protected function executePreFilters () {
		// Execute all pre filters
		$this->preRegistrationFilter->processFilters(FrameworkBootstrap::getRequestInstance(), FrameworkBootstrap::getResponseInstance());
	}

	/**
	 * Executes all post filters
	 *
	 * @return	void
	 */
	protected function executePostFilters () {
		// Execute all post filters
		$this->postRegistrationFilter->processFilters(FrameworkBootstrap::getRequestInstance(), FrameworkBootstrap::getResponseInstance());
	}

}
